/*
 ********** Begin Standard Header - Do not add comments here ***********
 * 
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 * 
 ************************** End Standard Header ***********************/

import java.util.*;
import java.text.SimpleDateFormat;
import com.tivoli.dms.api.proxy.*;
import com.tivoli.dms.api.*;
import java.net.URL;
public class ComplexExpressionSample implements ClientAPIConstants {
    //public static final String copyright = com.tivoli.dms.common.Copyright.COPYRIGHT;

    //This sample shows how to construct a Query for the following criteria - 
    //WHERE ( ( JOB_TYPE = 'DEVICE_CFG' AND ACTIVATION_TIME >= {ts '2003-04-01 00:00:00'} ) OR
    //        ( JOB_PRIORITY <= 3 AND JOB_STATUS = 'EXECUTABLE' ) )
    //It then searches for the jobs meeting the search criteria and displays the job ids.

    //It also shows how to implment ConstantsForAPIs so the constants can
    //be used in the code.

    //The date string must be in the following format: "2003-MM-DDThh:mm:ss GMT"

    //You may need to change the URL hostname and port of the DMS Server in the setEndPoint method
    //to match your environment.

    public static void main(String[] argList) {                 
        try {
            //construct job proxy
            JobManagerProxy proxy = new JobManagerProxy();
            proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/JobManagerService"));

            //construct query object
            Query query= new Query();

            //----
            //Create first query clause: job_type = 'DEVICE_CFG'
            //--
            QueryClause qc1 = new QueryClause();
            qc1.setAttribute(JOB_TYPE);
            qc1.setOperator(EQUAL);
            qc1.setValue(DMS__DEVICE_CONFIGURATION_JOB_TYPE);

            //----
            //Create second query clause: job_activation_date >= 01/01/2003
            //--
            QueryClause qc2 = new QueryClause();
            qc2.setAttribute(JOB_ACTIVATION_DATE);
            qc2.setOperator(GREATER_THAN_OR_EQUAL);

            //The date string  must be in the following format: "2003-04-01Thh:mm:ss GMT"
            GregorianCalendar cal = new GregorianCalendar(2003, Calendar.APRIL, 01);
            cal.set(Calendar.HOUR_OF_DAY, 0);
            cal.set(Calendar.MINUTE, 0);
            cal.set(Calendar.SECOND,0);
            cal.setTimeZone(TimeZone.getTimeZone("GMT"));
            SimpleDateFormat sdf  = new SimpleDateFormat("yyyy-MM-dd'T'H:mm:ss zz",                    Locale.getDefault());
            sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
            String date_str = sdf.format(cal.getTime());
            qc2.setValue(date_str);

            //create query expression
            QueryExpression qe1 = new QueryExpression();

            //add query clauses to query expression
            QueryClause[] clauses = {qc1, qc2};
            qe1.setQueryClauses(clauses);
            qe1.setCondition(AND);

            //---
            //Third query clause: job priority <= "3"
            //----
            QueryClause qc3 = new QueryClause();
            qc3.setAttribute(JOB_PRIORITY);
            qc3.setOperator(LESS_THAN_OR_EQUAL);
            qc3.setValue("3");

            //---
            //Fourth query clause: job status = "EXECUTABLE"
            //----
            QueryClause qc4 = new QueryClause();
            qc4.setAttribute(JOB_STATUS);
            qc4.setOperator(EQUAL);
            qc4.setValue(JOB_STATUS_EXECUTABLE);

            //---
            //Create QueryExpression and add query clauses to it
            //----
            QueryExpression qe2 = new QueryExpression(); 
            QueryClause[] clauses2 = {qc3, qc4};
            qe2.setQueryClauses(clauses2);
            qe2.setCondition(AND);
            
            //create another query expression and add two previous query expressions to it.
            QueryExpression ce = new QueryExpression();

            QueryExpression[] expressions = {qe1, qe2};
            ce.setQueryExpressions(expressions);
            ce.setCondition(OR);  //or the two expressions

            //add complex expression to query
            query.setQueryExpression(ce);

            //First get a count of jobs meeting criteria
            int count = proxy.countJobsFromQuery(query);
            System.out.println("Number of jobs found = " + count);

            //Get all jobs meeting criteria
            Job[] job_elements = proxy.getJobsFromQuery(query);

            //Display job id of each job meeting criteria
            for (int i=0; i<job_elements.length; i++) {
                Job job2 = (Job)job_elements[i];
                System.out.println("Job id from Query - " + job2.getJobID());
            }
            return;

        }catch (DeviceManagerException e) {
             System.out.println("API Exception  - " + e.getMessage());
        }catch (Exception ee) {
            System.out.println("Exception  - " + ee.getMessage());
        }
    }
}
