/*
 ********** Begin Standard Header - Do not add comments here ***********
 * 
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 * 
 ************************** End Standard Header ***********************/

import java.util.*;
import java.text.SimpleDateFormat;
import com.tivoli.dms.api.proxy.*;
import com.tivoli.dms.api.*;
import java.net.URL;
public class DeviceCfgJobSample implements ClientAPIConstants {
    //public static final String copyright = com.tivoli.dms.common.Copyright.COPYRIGHT;

    //This sample shows how to construct an Device Configuration job for a Wince device 
    //with the DEVICE_NAME of "wince1".  The device must already exist in the database.

    //It also shows how to implement ConstantsForAPIs so the constants can
    //be used in the code.

    public static void main(String[] argList) {                 
        try {
            //Create Proxies
            JobManagerProxy proxy = new JobManagerProxy();
            proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/JobManagerService"));
            DeviceManagerProxy device_proxy = new DeviceManagerProxy();
            device_proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/DeviceManagerService"));

            //Get device id from name
            Device device = device_proxy.getDeviceFromName("wince1", "Wince");

            //Setup job. 
            Job job1 = new Job();
            job1.setDeviceClassName("Wince");
            job1.setJobType(DMS__DEVICE_CONFIGURATION_JOB_TYPE);
            job1.setJobPriority(2);
            job1.setJobDescription("my new job");
            job1.setExpirationTime(new GregorianCalendar(2006, Calendar.JANUARY, 1));
            
            //Set job parms
            HashMap cfg_parms = new HashMap();
            cfg_parms.put("pop3.server", "http://mypopserver");   
            cfg_parms.put("pop3.uid", "frank");        
            job1.setJobParmsMap(cfg_parms);

            //Set target device
            long target_devices[] ={device.getDeviceID()};
            job1.setDevicesForJob(target_devices);

            //Create the job and return the jobID
            long job_id = proxy.createJob(job1);
            System.out.println("Job submitted - job id = " + job_id);

            return;
        }catch (DeviceManagerException se) {
            System.out.println("API Exception - " + se.getMessage());
        }catch (Exception e) {
            System.out.println("Exception - " + e);
        }
    }
}
