/*
 ********** Begin Standard Header - Do not add comments here ***********
 * 
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 * 
 ************************** End Standard Header ***********************/

import java.lang.*;
import java.text.SimpleDateFormat;
import java.util.*;
import java.text.*;
import java.io.*;
import com.tivoli.dms.api.*;
import com.tivoli.dms.api.proxy.*;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;
import java.net.URL;

//This sample shows how to use the DeviceJobProxy to retrieve status information on a job
//and it's target devices.

//This sample will take an input parameter consisting of a job_id. If not passed in,
//it will create a Wince device and submit an inventory job to it. 

public class DeviceJobSample  implements ClientAPIConstants{
    //public static final String copyright = com.tivoli.dms.common.Copyright.COPYRIGHT;
  public static void main(String[] argList) {                 
    Job job, job1;
    long jobid = -1;

    try {
        //Create Proxies
        JobManagerProxy job_proxy = new JobManagerProxy();
        job_proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/JobManagerService"));

        DeviceManagerProxy device_proxy = new DeviceManagerProxy();
        device_proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/DeviceManagerService"));

        DeviceJobManagerProxy devicejob_proxy = new DeviceJobManagerProxy();
        devicejob_proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/DeviceJobManagerService"));

        //---------------------------------------------------------------------------------
        //If job_id not passed - create a device and submit a job to it
        //----------------------------------------------------------------------------------
        if (argList!=null && argList.length>0) {
            String jobstr = argList[0];
            jobid = Long.valueOf(jobstr).longValue();
        }

        //Create device and job if not passed in
        if (jobid == -1) {
           //Create device
           Device device = new Device();
           device.setDeviceClassName("Wince");
           device.setDeviceName("wince1");
           device.setDeviceOwner("frank");
           long device_id = device_proxy.createDevice(device);

           //Now submit job to device
           job1 = new Job();
           job1.setDeviceClassName("Wince");
           job1.setJobType("INVENTORY");
           job1.setJobPriority(3);
           job1.setExpirationTime(new GregorianCalendar(2008, Calendar.JANUARY, 1));

           //job parms
           HashMap map = new HashMap();
           map.put(DMS__INV_SCAN_JOB_PARM, "Configuration,Hardware,Software");   
           job1.setJobParmsMap(map);

           //Job is for existing devices and the one we just created 
           job1.setJobTargetScope(FOR_EXISTING_AND_NEW_DEVICES);
           long[] device_ids = {device_id};
           job1.setDevicesForJob(device_ids);

           //create Job now
           jobid = job_proxy.createJob(job1);
           System.out.println("Job created, id = " + jobid);
        }

        //-----------------------------------------------------
        //get all of  the job's target devices
        //--------------------------------------------------------
        System.out.println("\nGet job, id = " + jobid);
        job1 = job_proxy.getJob(jobid);
        long[] dev_ids = job1.getDevicesForJob();

        //Get DeviceJob for each target device
        //--
        for (int i=0;i<dev_ids.length;i++) {
           System.out.println("\nget DeviceJob for " + jobid  + ":" +dev_ids[i]); 
           DeviceJob devjob = devicejob_proxy.getDeviceJob(jobid,dev_ids[i]);
           System.out.println("\nDeviceJob: ");
           System.out.println(" jobID - " + devjob.getJobID());
           System.out.println(" jobType - " + devjob.getJobType());
           System.out.println(" jobDescription - " + devjob.getJobDescription());
           System.out.println(" deviceClassName - " + devjob.getDeviceClassName());
           System.out.println(" deviceID - " + devjob.getDeviceID());
           System.out.println(" deviceName - " + devjob.getDeviceName());
           System.out.println(" deviceOwner - " + devjob.getDeviceOwner());
           System.out.println(" jobStatus - " + devjob.getJobStatus());
           System.out.println(" lastcompletionStatus - " + devjob.getLastCompletionStatus());
           System.out.println(" lastcompletionTime - " + devjob.getLastCompletionTime());
           System.out.println(" completionMessage - " + devjob.getCompletionMessage());
           System.out.println(" nextJobRunTime - " + devjob.getNextJobRunTime());
        }

        //------------------------------------
        //getJobCompletionStatusValues
        //------------------------------------
        String[] attr_names = devicejob_proxy.getDeviceJobCompletionValues();
        System.out.println("\nDeviceJobCompletionValues:");
        for (int i=0; i<attr_names.length; i++) {
          System.out.println(attr_names[i]);
        }

        //-------------------
        //getQueryAttributes
        //------------------
         attr_names = devicejob_proxy.getQueryAttributeNames();
        System.out.println("\nQueryAttributeNames:");
        for (int i=0; i<attr_names.length; i++) {
          System.out.println(attr_names[i]);
        }


        //--------------------------------------
        //getDeviceJobStatus
        //--------------------------------------
        System.out.println("\nDeviceJobStatus:");
        Element element = devicejob_proxy.getDeviceJobStatus(jobid,-1);
        //Element element = proxy2.getDeviceJobStatus(jobid,device_id);
        StringWriter  stringOut = new StringWriter();        //Writer will be a String
        OutputFormat outFormat = new OutputFormat();
        outFormat.setIndenting(true);
        outFormat.setEncoding("UTF-8");
        XMLSerializer serial = new XMLSerializer();
        serial.setOutputCharStream(stringOut);
        serial.setOutputFormat(outFormat);
        serial.asDOMSerializer();                            // As a DOM Serializer
        serial.serialize( element);
        System.out.println( "STRXML = " + stringOut.toString() ); //Spit out DOM as a String

        //-----------------------------------------
        //getDeviceJobSummary
        //-----------------------------------------
        System.out.println("\nDeviceJobSummary:");
        element = devicejob_proxy.getDeviceJobSummary(jobid);
        stringOut = new StringWriter();        //Writer will be a String
        outFormat = new OutputFormat();
        outFormat.setIndenting(true);
        outFormat.setEncoding("UTF-8");
        serial = new XMLSerializer();
        serial.setOutputCharStream(stringOut);
        serial.setOutputFormat(outFormat);
        serial.asDOMSerializer();                            // As a DOM Serializer
        serial.serialize( element);
        System.out.println( "STRXML = " + stringOut.toString() ); //Spit out DOM as a String

        return;
    }catch (DeviceManagerException se) {
        System.out.println("API Exception - " + se.getMessage());
    } catch (Exception e) {
        System.out.println("Exception - " + e);
    }
}
}
