/*
 ********** Begin Standard Header - Do not add comments here ***********
 * 
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 * 
 ************************** End Standard Header ***********************/

import java.util.*;
import java.io.*;
import java.text.SimpleDateFormat;
import com.tivoli.dms.api.*;
import com.tivoli.dms.api.proxy.*;
import java.net.URL;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;
public class DumpJobParmsSample implements ClientAPIConstants {
    //public static final String copyright = com.tivoli.dms.common.Copyright.COPYRIGHT;

   //This sample shows how to list information about job parameters for a device class.
   //The device class name is passed as a parameter. It uses "BaseOMADM" as the default
   //if one is not passsed in.

   //It also shows how to implement ConstantsForAPIs so the constants can
   //be used in the code.

    public static void main(String[] argList) {                 
        try {
            //Get device class name
           String device_class_name = "BaseOMADM";
           if (argList.length>0) {
              device_class_name = argList[0];
           }
           System.out.println("Device class name = " + device_class_name);

            //Create Proxies
            JobManagerProxy job_proxy = new JobManagerProxy();
            job_proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/JobManagerService"));

            //-----------------------
            //Test getJobTypes
            //-------------------------
            String[] job_types = job_proxy.getJobTypes(device_class_name);
            for (int i=0; i<job_types.length; i++) {
              System.out.println("\nJobType = " + job_types[i]);

              //System.out.println("Job parm keys:");
              String[]job_parm_keys = job_proxy.getJobParmKeys(device_class_name, job_types[i]);
              for (int j=0; j<job_parm_keys.length; j++) {
                //System.out.println(job_parm_keys[j]);
                System.out.println("\nJobType = " + job_types[i] + " Job parm key = " + job_parm_keys[j]);
                String choices[] = job_proxy.getJobParmChoices(job_parm_keys[j], device_class_name, job_types[i], -1);
                if (choices.length>0) {
                   ArrayList al = new ArrayList(Arrays.asList(choices));
                   System.out.println("Job parm value choices = " + al);
                } 

              }

              Element element1 = job_proxy.getJobParmMetaData(device_class_name, job_types[i]);
              StringWriter stringOut = new StringWriter();        //Writer will be a String
              OutputFormat outFormat = new OutputFormat();
              outFormat.setIndenting(true);
              outFormat.setEncoding("UTF-8");
              XMLSerializer serial = new XMLSerializer();
              serial.setOutputCharStream(stringOut);
              serial.setOutputFormat(outFormat);
              serial.asDOMSerializer();                            // As a DOM Serializer
              serial.serialize( element1);
              System.out.println( "\nJobParmMetaData for job type - " + job_types[i]+ ":");
              System.out.println(stringOut.toString() ); //Spit out DOM as a String
            }


            return;
        }catch (DeviceManagerException se) {
            System.out.println("API Exception - " + se.getMessage());
        } catch (Exception e) {
            System.out.println("Exception  - " + e);
        }
    }
}
