/*
 ********** Begin Standard Header - Do not add comments here ***********
 * 
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 * 
 ************************** End Standard Header ***********************/

import java.util.*;
import java.io.StringWriter;
import java.text.SimpleDateFormat;
import com.tivoli.dms.api.proxy.*;
import com.tivoli.dms.api.*;
import java.net.URL;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;
public class InventoryParseSample implements ClientAPIConstants {
    //public static final String copyright = com.tivoli.dms.common.Copyright.COPYRIGHT;

    //This sample shows how to parse Inventory XML, print the xml and print the table and columns.
    //Inventory must exist in the database for a Wince device with the DEVICE_NAME of "wince1".

    //This sample uses the xerces DOM parser to parse the xml.

    //It also shows how to implement ConstantsForAPIs so the constants can
    //be used in the code.

    // You may need to change the URL hostname and port of the DMS Server in the setEndPoint method
    // to match your environment.

    public static void main(String[] argList) {                 
        try {
            //Create Proxy
            DeviceManagerProxy device_proxy = new DeviceManagerProxy();
            device_proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/DeviceManagerService"));

            //Get device id from name
            Device device = device_proxy.getDeviceFromName("wince1", "Wince");

            //Get Device inventory
            Element element = device_proxy.getDeviceInventory(device.getDeviceID());

            //-------------------------------
            //Print device inventory xml
            //------------------------------
            StringWriter stringOut = new StringWriter();        //Writer will be a String
            OutputFormat outFormat = new OutputFormat();
            outFormat.setIndenting(true);
            outFormat.setEncoding("UTF-8");
            XMLSerializer serial = new XMLSerializer();
            serial.setOutputCharStream(stringOut);
            serial.setOutputFormat(outFormat);
            serial.asDOMSerializer();                            // As a DOM Serializer
            serial.serialize( element);
            System.out.println( "STRXML = " + stringOut.toString() ); //Spit out DOM as a String

            //-----------------------------------------------------
            //Parse XML and print Table, column name and column data
            //------------------------------------------------------
            String column_name, column_value;

            //Get table elements
            NodeList tables = element.getChildNodes();
            if (tables != null) {
                for (int i = 0; i < tables.getLength(); i++) { //For each table element
                    //table element
                    Element tab_element = (Element) tables.item(i);
                    //Print table name
                    System.out.println("\nTable = " + tab_element.getTagName());

                    //Get record elements
                    NodeList record_list = tab_element.getElementsByTagName("Record");
                    if (record_list != null && record_list.getLength()>0) {
                        for (int j = 0; j < record_list.getLength(); j++) { //For all Records
                            //Record element
                            Element record_element = (Element) record_list.item(j);
                            System.out.println("==> Record " + (j+1));

                            //Get record columns
                            NodeList columns_list = record_element.getChildNodes();
                            if (columns_list != null && columns_list.getLength()>0) {
                                for (int k = 0; k < columns_list.getLength(); k++) { //For each column
                                    //column element
                                    Element col_element = (Element)columns_list.item(k);
                                    //column name
                                    column_name =  col_element.getTagName();

                                    //Get column value. It may not be there.
                                    Node value_element = col_element.getFirstChild();
                                    if (value_element!=null) {
                                       column_value = value_element.getNodeValue().trim();
                                    } else
                                       column_value = "";
                                    System.out.println("====> Column = " + column_name + " Value = " + column_value);
                                }
                            }
                        }
                    }
                }
            }

            return;
        }catch (DeviceManagerException se) {
            System.out.println("API Exception - "  + se.getMessage());
        }catch (Exception e) {
            System.out.println("Exception - " + e);
        }
    }
}
