/*
 ********** Begin Standard Header - Do not add comments here ***********
 * 
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 * 
 ************************** End Standard Header ***********************/

import java.lang.*;
import java.text.SimpleDateFormat;
import java.util.*;
import java.text.*;
import java.io.*;
import com.tivoli.dms.api.proxy.*;
import com.tivoli.dms.api.*;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;
import java.net.URL;

//This sample shows how to build various types of Queries and use them to search for devices.
//directly. The last example also shows how to submit a job a query.

public class QuerySample  implements ClientAPIConstants{
    //public static final String copyright = com.tivoli.dms.common.Copyright.COPYRIGHT;
  public static void main(String[] argList) {                 
    try {

        //Create Proxies
        JobManagerProxy job_proxy = new JobManagerProxy();
        job_proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/JobManagerService"));

        SoftwareManagerProxy sw_proxy = new SoftwareManagerProxy();
        sw_proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/SoftwareManagerService"));

        DeviceManagerProxy device_proxy = new DeviceManagerProxy();
        device_proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/DeviceManagerService"));

        NamedQueryManagerProxy nq_proxy = new NamedQueryManagerProxy();
        nq_proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/NamedQueryManagerService"));

        //---------------------------------------------------------------------------------
        //This shows how to create a Query consisting of a named query and use
        //it to get a count of devices
        //----------------------------------------------------------------------------------
        Query q = new Query();
        QueryClause qc = new QueryClause();
        qc.setAttribute("QUERY_NAME");
        qc.setOperator(EQUAL);
        qc.setValue("All Wince Devices");
        q.setQueryClause(qc);
        device_proxy.countDevicesFromQuery(q);
        System.out.println("\nCount of All Wince devices = " + device_proxy.countDevicesFromQuery(q));

        //---------------------------------------------------------------------------------
        //This query shows how to search based on inventory data.
        //Check for all devices running the Windows CE Operating System.
        //----------------------------------------------------------------------------------
        q = new Query();
        qc = new QueryClause();
        qc.setAttribute("COMPUTER.OS_NAME");
        qc.setOperator(EQUAL);
        qc.setValue("~Windows CE");
        q.setQueryClause(qc);
        device_proxy.countDevicesFromQuery(q);
        System.out.println("\nCount of devices running 'Windows CE' = " + device_proxy.countDevicesFromQuery(q));
        Device[] devices = device_proxy.getDevicesFromQuery(q);
        for (int i=0;i<devices.length;i++) {
           Device device = devices[i];
           System.out.println("Device class = " + device.getDeviceClassName() + " Device name = " + device.getDeviceName());
        } 


        //---------------------------------------------------------------------------------
        //This shows how to list the base device attribute names.
        //----------------------------------------------------------------------------------
        String[] attr_names = device_proxy.getQueryAttributeNames(null);
        System.out.println("\nBase device attribute names");
        for (int i=0;i<attr_names.length;i++) {
           System.out.println(attr_names[i]);
        } 

        //---------------------------------------------------------------------------------
        //This query shows how to search for devices based on notification data
        //----------------------------------------------------------------------------------
        q = new Query();
        qc = new QueryClause();
        qc.setAttribute("NOTIFICATION.NETWPIN");
        qc.setOperator(EQUAL);
        qc.setValue("netdata");
        q.setQueryClause(qc);
        device_proxy.countDevicesFromQuery(q);
        System.out.println("\nCount of devices with notification attribute = " + device_proxy.countDevicesFromQuery(q));
        devices = device_proxy.getDevicesFromQuery(q);
        for (int i=0;i<devices.length;i++) {
           Device device = devices[i];
           System.out.println("Device class = " + device.getDeviceClassName() + " Device name = " + device.getDeviceName());
        } 

        //--------------------------------------------------------------------------------------
        //This query shows how to query the INSTALLED_SW table.
        //Check for all devices that have a specific software package (based on sw_id) already installed on it.
        //--------------------------------------------------------------------------------------
        q = new Query();
        qc = new QueryClause();
        qc.setAttribute("INSTALLED_SW.SW_ID");
        qc.setValue("40427200341612120");
        qc.setOperator(EQUAL);
        q.setQueryClause(qc);
        System.out.println("\nCount of devices with Software package installed = " + device_proxy.countDevicesFromQuery(q));
        devices = device_proxy.getDevicesFromQuery(q);
        for (int i=0;i<devices.length;i++) {
           Device device = devices[i];
           System.out.println("==> Device class = " + device.getDeviceClassName() + " Device name = " + device.getDeviceName());
        } 

        //-------------------------------------------------------------------------------------------------
        //This query shows how to use the "NOT_EXISTS" condition to test that the query result is false.
        //Get all devices that DO NOT have a software package (based on sw_id) installed on it.
        //-------------------------------------------------------------------------------------------------
        q = new Query();
        qc = new QueryClause();
        qc.setAttribute("INSTALLED_SW.SW_ID");
        qc.setValue("40427200341612120");
        qc.setOperator(EQUAL);

        QueryExpression qe = new QueryExpression();
        QueryClause[] qcs = {qc};
        qe.setCondition(NOT_EXISTS);
        qe.setQueryClauses(qcs);

        q.setQueryExpression(qe);
        System.out.println("\n1. Count of devices without Software package (sw_id) installed = " + device_proxy.countDevicesFromQuery(q));
        devices = device_proxy.getDevicesFromQuery(q);
        for (int i=0;i<devices.length;i++) {
           Device device = devices[i];
           System.out.println("==> Device class = " + device.getDeviceClassName() + " Device name = " + device.getDeviceName());
        } 

        //-------------------------------------------------------------------------------------------------
        //This query shows how to explicitly specify the where clause.
        //Submit a job to all devices that DO NOT have a software package (based on sw_id) installed on it.
        //We are using a special query clause in which the where clause to be used to search for devices is explicitly specified.
        //-------------------------------------------------------------------------------------------------
        q = new Query();
        qc = new QueryClause();
        qc.setAttribute("LEAVE_UNALTERED");
        qc.setValue("DEVICE.device_id in (select device_id from device a where not exists (select 1 from installed_sw b where a.device_id=b.device_id and b.sw_id=40427200341612120)) ");

        qc.setOperator("=");
        q.setQueryClause(qc);
        System.out.println("\n2. Count of devices without Software package (sw_id) installed = " + device_proxy.countDevicesFromQuery(q));
        devices = device_proxy.getDevicesFromQuery(q);
        for (int i=0;i<devices.length;i++) {
           Device device = devices[i];
           System.out.println("==> Device class = " + device.getDeviceClassName() + " Device name = " + device.getDeviceName());
        } 
        
        //-------------------------------------------------------------------------------------------------
        //This shows how to submit a job using above query to target devices
        //-------------------------------------------------------------------------------------------------
        Job job1 = new Job();

        //target job to query
        job1.setQuery(q);

        job1.setDeviceClassName("Wince");
        job1.setJobType(DMS__DEVICE_CONFIGURATION_JOB_TYPE);
        job1.setJobPriority(2);
        job1.setJobDescription("device cfg to all device with sw installed");

        //Set job parms
        HashMap cfg_parms = new HashMap();
        cfg_parms.put("pop3.server", "http://mypopserver");   
        cfg_parms.put("pop3.uid", "frank");        
        job1.setJobParmsMap(cfg_parms);

        //Create the job and return the jobID
        long job_id = job_proxy.createJob(job1);
        System.out.println("\nJob submitted to Query, job id =" + job_id);

        //-------------------------------------------------------------------------------------------------
        //This shows how to search for jobs based on a query of job parms
        //-------------------------------------------------------------------------------------------------
        q = new Query();
        qc = new QueryClause();
        qc.setAttribute("JOB_PARM.ppp.user");
        qc.setValue("1");
        qc.setOperator("=");
        q.setQueryClause(qc);
        System.out.println("\nCount of jobs with a particular job parm configured = " + job_proxy.countJobsFromQuery(q));
        Job[] jobs = job_proxy.getJobsFromQuery(q);
        for (int i=0;i<jobs.length;i++) {
           Job job = jobs[i];
           System.out.println("Job id = " + job.getJobID() + " Job type = " + job.getJobType());
        } 

        //---------------------------------------------------------------------------------
        //This shows how to create a NamedQuery and use it in a query to search for devices.
        //----------------------------------------------------------------------------------
        //First create a Query
        q = new Query();
        qc = new QueryClause();
        qc.setAttribute(DEVICE_CLASS_NAME);
        qc.setOperator(EQUAL);
        qc.setValue("Wince");

        QueryClause qc2 = new QueryClause();
        qc2.setAttribute(DEVICE_OWNER);
        qc2.setOperator(EQUAL);
        qc2.setValue("frank");

        qe = new QueryExpression();
        QueryClause[] qcs2 = {qc, qc2};

        qe.setQueryClauses(qcs2);
        qe.setCondition(AND);

        q.setQueryExpression(qe);
        
        //Next create a named query using above query
        NamedQuery nq = new NamedQuery();
        nq.setQueryName("My Wince Devices");
        nq.setDescription("All devices owned by frank");
        nq.setQuery(q);
        long nq_id = nq_proxy.createNamedQuery(nq);
        System.out.println("\nNamedQuery created, query_id = " + nq_id);

        //Now set up a Query containing named query
        q = new Query();
        qc = new QueryClause();
        qc.setAttribute("QUERY_NAME");
        qc.setOperator(EQUAL);
        qc.setValue("My Wince Devices");

        q.setQueryClause(qc);

        //Now use Query containing named query to search for devices
        devices = device_proxy.getDevicesFromQuery(q);
        System.out.println("\nNumber of devices owned by me = " + devices.length);

        //Now delete the named query
        nq_proxy.deleteNamedQuery(nq_id);
        System.out.println("\nNamedQuery deleted");

        return;
    }catch (DeviceManagerException se) {
        System.out.println("API Exception - " + se.getMessage());
    } catch (Exception e) {
        System.out.println("Exception - " + e);
    }
}
}
