/*
 ********** Begin Standard Header - Do not add comments here ***********
 * 
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 * 
 ************************** End Standard Header ***********************/

import java.util.*;
import java.text.SimpleDateFormat;
import com.tivoli.dms.api.proxy.*;
import com.tivoli.dms.api.*;
import java.net.URL;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;
import org.apache.xerces.dom.DocumentImpl;
public class SyncMLJobsSample implements ClientAPIConstants {
    //public static final String copyright = com.tivoli.dms.common.Copyright.COPYRIGHT;

    //This sample shows how to construct various SyncML jobs. 
    //It requires an existing device with the DEVICE_NAME of "sync1".
    //The device must already exist in the database.

    //It also shows how to implement ConstantsForAPIs so the constants can
    //be used in the code.

    public static void main(String[] argList) {                 
        try {
            //Create Proxies
            JobManagerProxy proxy = new JobManagerProxy();
            proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/JobManagerService"));
            DeviceManagerProxy device_proxy = new DeviceManagerProxy();
            device_proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/DeviceManagerService"));

            //Get device id from name. Create it if it doesn't exist
            Device device;
            try {
               device = device_proxy.getDeviceFromName("sync1", "BaseOMADM");
            } catch (Exception se) {
               device = new Device();
               device.setDeviceName("sync1");
               device.setDeviceClassName("BaseOMADM");
               device.setDeviceOwner("fbaldiga");
               device.setDeviceDescription("sync1");

               //set device class attributes
               HashMap map2 = new HashMap();
               map2.put("DEVICE_USERNAME","jones");
               map2.put("DEVICE_PASSWORD","password");
               map2.put("SERVER_ID","frankb");
               map2.put("SERVER_PASSWORD","password");
               device.setDeviceClassAttributesMap(map2);

               device_proxy.createDevice(device);
               device = device_proxy.getDeviceFromName("sync1", "BaseOMADM");
               System.out.println("Device created - sync1");
            }

            //******************************************************
            //Setup for Node Discovery job. 
            //******************************************************
            Job job1 = new Job();
            job1.setDeviceClassName("BaseOMADM");
            job1.setJobPriority(2);
            job1.setJobDescription("my new job");
            job1.setExpirationTime(new GregorianCalendar(2008, Calendar.JANUARY, 1));
            job1.setJobType("SYNCMLDM_WTREE");
            job1.setJobType(ClientAPIConstants.DMS__NODE_DISCOVERY_JOB_TYPE);
            job1.setJobDescription("Node Discovery job for device sync1");
            
            //Set job parms
            HashMap cfg_parms = new HashMap();
            cfg_parms.put("TREE_WALKER_TARGET_URI",".");   
            job1.setJobParmsMap(cfg_parms);

            //Set target device
            long target_devices[] ={device.getDeviceID()};
            job1.setDevicesForJob(target_devices);

            //Create the job and return the jobID
            long job_id = proxy.createJob(job1);
            System.out.println("Node Discovery job submitted - job id = " + job_id);

            //******************************************************
            //Custom Command job
            //Submit job to change language and other items
            //******************************************************
            job1 = new Job();
            HashMap job_parms = new HashMap();

            //While not required, when you only have multiple commands of the same
            //type as in this example (Replace cmds), you should get in the habit of
            //also including the CMD_NUMBER key for each group.  This defines the
            //order in which the commands are sent to the device.
            job_parms.put("1#REPLACE_CMD_NUMBER", "1");  

            //The CMD_GROUP key tells if commands are:
            // 0 = not grouped
            // 1 = Atomic - Commands in group must be processed as a set or not at all (transaction). 
            //     Get and Exec commands are not permitted in an atomic set of commands. 
            // 2 = Sequence - more than one command type must be specified and these commands
            //     must be processed in the specified sequence. 
            job_parms.put("1#REPLACE_CMD_GROUP", "0");

            //Command Grouping is done by prefixing a 1 based index number and the pound sign (#)
            //on the keys.  
            job_parms.put("1#REPLACE_ITEM_1_TARGET_URI", "./Test/DisplayLanguage");
            job_parms.put("1#REPLACE_ITEM_1_DATA", "4");          //4 = French

            job_parms.put("2#REPLACE_CMD_NUMBER", "2");

            //This command (2#) and the next one (3#) are atomic (transaction)
            job_parms.put("2#REPLACE_CMD_GROUP", "1");

            job_parms.put("2#REPLACE_ITEM_1_TARGET_URI", "./Test/Color");
            job_parms.put("2#REPLACE_ITEM_1_DATA", "red");

            //When we get to sending binary data (image/gif) it will be a little different.
            //The data will have to be read from a file like and the job parms would be - 
            job_parms.put("3#REPLACE_CMD_NUMBER", "3");
            job_parms.put("3#REPLACE_CMD_GROUP", "1");
            job_parms.put("3#REPLACE_ITEM_1_TARGET_URI", "./Test/background");
            job_parms.put("3#REPLACE_ITEM_1_DATA", "http://localhost/masthead.gif");
            job_parms.put("3#REPLACE_ITEM_1_ALTDATASOURCE", "bin from URL");

            job1.setDeviceClassName("BaseOMADM");
            job1.setJobType("SYNCMLDM_CMD");
            job1.setJobPriority(2);
            job1.setJobDescription("Change language");
            job1.setExpirationTime(new GregorianCalendar(2008, Calendar.JANUARY, 1));
            job1.setJobParmsMap(job_parms);
                        
            //Set target device          
            job1.setDevicesForJob(target_devices);

            //Create the job and return the jobID
            job_id = proxy.createJob(job1);      
            System.out.println("Custom Command job submitted - job id = " + job_id);

            //****************************************************************************
            // Custom Command Job to retrieve the specific OMADM Nodes that contain the Software version and 
            // Hardware version from the device. We then parse the SYNCDM_TREE table where the results are stored.
            // Note that in production, the job must complete first which causes the data to be retrieved
            // from the device and stored in the database before parsing the SYNDMC_TREE table where the
            // results are stored.
            //*****************************************************************************
            job1 = new Job();

            //This job performs two get commands which stores the results in the SYNCDM_TREE table
            cfg_parms = new HashMap();
            cfg_parms.put("1#GET_CMD_NUMBER", "1");  
            cfg_parms.put("1#GET_CMD_GROUP", "0");
            cfg_parms.put("1#GET_ITEM_1_TARGET_URI", "./DevDetail/HwV");
            cfg_parms.put("2#GET_CMD_NUMBER", "2");  
            cfg_parms.put("2#GET_CMD_GROUP", "0");
            cfg_parms.put("2#GET_ITEM_1_TARGET_URI", "./DevDetail/SwV");

            job1.setDeviceClassName("BaseOMADM");
            job1.setJobType("SYNCMLDM_CMD");
            job1.setJobPriority(2);
            job1.setJobDescription("Get Hw and Sw Version");
            job1.setExpirationTime(new GregorianCalendar(2008, Calendar.JANUARY, 1));
            job1.setJobParmsMap(cfg_parms);
                        
            //Set target device          
            job1.setDevicesForJob(target_devices);

            //Create the job and return the jobID
            job_id = proxy.createJob(job1);      
            System.out.println("Custom Get Command job submitted - job id = " + job_id);

            //Get Hardware and Software version fromn SYNCDM_TREE database table
            Element device_element = device_proxy.getDeviceInventoryView(device.getDeviceID(), "SYNCDM_TREE");

            //Parse SYNCDM_TREE xml and get hardware and Software version nodes
            String column_name, column_value, uri_value, data_value;
            TreeMap data_map = new TreeMap();
            NodeList tables = device_element.getChildNodes();
            if (tables != null) {
               //SYNCDM_TREE element
               Element tab_element = (Element) tables.item(0);

               //Get record elements
               NodeList record_list = tab_element.getElementsByTagName("Record");
               if (record_list != null && record_list.getLength()>0) {
                  for (int j = 0; j < record_list.getLength(); j++) { //For all Records
                     //Record element
                     Element record_element = (Element) record_list.item(j);
                     //out.println("==> Record " + (j+1));
                     uri_value = null; data_value=null;

                     //Get URI element
                     NodeList column_list = record_element.getElementsByTagName("URI");
                     if (column_list!=null && column_list.getLength()>0) {
                        Element col_element = (Element)column_list.item(0);
                        //Get URL value. It may not be there.
                        Node value_element = col_element.getFirstChild();
                        if (value_element!=null) 
                           uri_value = value_element.getNodeValue().trim();
                     }
                     //Get DATA element
                     column_list = record_element.getElementsByTagName("DATA");
                     if (column_list!=null && column_list.getLength()>0) {
                        Element col_element = (Element)column_list.item(0);
                        //Get URL value. It may not be there.
                        Node value_element = col_element.getFirstChild();
                        if (value_element!=null) 
                           data_value = value_element.getNodeValue().trim();
                     }
                     //print out hardware and software version
                     if (uri_value!=null && uri_value.equals("./DevDetail/HwV")) {
                        System.out.println(uri_value + " = " + data_value);
                     }  else if (uri_value!=null && uri_value.equals("./DevDetail/SwV")) {
                        System.out.println(uri_value + " = " + data_value);
                     }

                  }
               }
            }

            //******************************************************
            //Notification job
            //******************************************************
            job1 = new Job();
            job1.setDeviceClassName("BaseOMADM");
            job1.setJobType("NOTIFICATION");
            job1.setJobPriority(2);
            job1.setJobDescription("Notification Job");
            job1.setExpirationTime(new GregorianCalendar(2008, Calendar.JANUARY, 1));
            job_parms = new HashMap();
            job1.setJobParmsMap(job_parms);
                        
            //Set target device          
            job1.setDevicesForJob(target_devices);

            //Create the job and return the jobID
            job_id = proxy.createJob(job1);      
            System.out.println("Notification job submitted - job id = " + job_id);


            //******************************************************
            //Bootstrap job
            //******************************************************
            job1 = new Job();
            job1.setDeviceClassName("BaseOMADM");
            job1.setJobType("BOOTSTRAP");
            job1.setJobPriority(2);
            job1.setJobDescription("Bootstrap device with account=newAccount security=USERPIN");
            job1.setExpirationTime(new GregorianCalendar(2008, Calendar.JANUARY, 1));

            //Set job parms
            job_parms = new HashMap();
            job_parms.put("BOOT_PARM_DMACCOUNT", "newAccount");
            job_parms.put("BOOT_PARM_SECURITY", "USERPIN");
            job1.setJobParmsMap(job_parms);

            //force this ON
            job1.setSendNotification("T");
                        
            //Set target device          
            job1.setDevicesForJob(target_devices);

            //Create the job and return the jobID
            job_id = proxy.createJob(job1);      
            System.out.println("Bootstrap job submitted - job id = " + job_id);


            //****************************************************************************
            //Command Script job - assume here is a command script at http://localhost/sample1.script
            //****************************************************************************
            job1 = new Job();
            job1.setDeviceClassName("BaseOMADM");
            job1.setJobType("SYNCMLDM_SCRIPT");
            job1.setJobPriority(2);
            job1.setJobDescription("Command script (http://localhost/sample1.script)");
            job1.setExpirationTime(new GregorianCalendar(2008, Calendar.JANUARY, 1));

            //Set job parms
            job_parms = new HashMap();
            job_parms.put("SCRIPT_URL","http://localhost/sample1.script");   
            job1.setJobParmsMap(job_parms);

            //force this ON
            job1.setSendNotification("T");
                        
            //Set target device          
            job1.setDevicesForJob(target_devices);

            //Submit Command Script Job 
            job_id = proxy.createJob(job1);      
            System.out.println("Command script job submitted - job id = " + job_id);


            //******************************************************
            //Get Device Inventory - Dump the SYNCDM_TREE database table
            //******************************************************
            System.out.println("Get device inventory");
            device_element = device_proxy.getDeviceInventoryView(device.getDeviceID(), "SYNCDM_TREE");
            data_map = new TreeMap();

            //Parse SYNCDM_TREE xml and Store URI and Data in HashMap
            tables = device_element.getChildNodes();
            if (tables != null) {
               //SYNCDM_TREE element
               Element tab_element = (Element) tables.item(0);

               //Get record elements
               NodeList record_list = tab_element.getElementsByTagName("Record");
               if (record_list != null && record_list.getLength()>0) {
                  for (int j = 0; j < record_list.getLength(); j++) { //For all Records
                     //Record element
                     Element record_element = (Element) record_list.item(j);
                     //out.println("==> Record " + (j+1));
                     uri_value = null; data_value=null;

                     //Get URI element
                     NodeList column_list = record_element.getElementsByTagName("URI");
                     if (column_list!=null && column_list.getLength()>0) {
                        Element col_element = (Element)column_list.item(0);
                        //Get URL value. It may not be there.
                        Node value_element = col_element.getFirstChild();
                        if (value_element!=null) 
                           uri_value = value_element.getNodeValue().trim();
                     }
                     //Get DATA element
                     column_list = record_element.getElementsByTagName("DATA");
                     if (column_list!=null && column_list.getLength()>0) {
                        Element col_element = (Element)column_list.item(0);
                        //Get URL value. It may not be there.
                        Node value_element = col_element.getFirstChild();
                        if (value_element!=null) 
                           data_value = value_element.getNodeValue().trim();
                     }

                     //Store in hashmap
                     if (uri_value!=null && data_value!=null) {
                        data_map.put(uri_value, data_value);
                     }
                  }
               }
            }
            Iterator itr = data_map.entrySet().iterator();
            System.out.println("\nSYNCDM_TREE table");
            System.out.println("-----------------");
            while (itr.hasNext()) {
               Map.Entry e = (Map.Entry)itr.next();
               String uri = (String)e.getKey(); 
               String data = (String)e.getValue();
               System.out.println(uri + " : " + data);
            }


            return;
        }catch (DeviceManagerException se) {
            System.out.println("API Exception - " + se.getMessage());
        }catch (Exception e) {
            System.out.println("Exception - " + e);
        }
    }
}
