/*
 ********** Begin Standard Header - Do not add comments here ***********
 * 
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 * 
 ************************** End Standard Header ***********************/

import java.util.*;
import java.io.*;
import java.text.SimpleDateFormat;
import com.tivoli.dms.api.proxy.*;
import com.tivoli.dms.api.*;
import java.net.URL;
import org.w3c.dom.*;
import org.apache.xml.serialize.*;
import org.apache.xerces.dom.DocumentImpl;
public class Win32NotificationSample implements ClientAPIConstants {
    //public static final String copyright = com.tivoli.dms.common.Copyright.COPYRIGHT;

    //This sample shows how to call various Notification Attribute-related methods. 

    //It also shows how to implement ConstantsForAPIs so the constants can
    //be used in the code.

    public static void main(String[] argList) {                 
        try {
            //Create Proxies
            JobManagerProxy proxy = new JobManagerProxy();
            proxy.setEndPoint(new URL("http://localhost:9080/dmserver/services/JobManagerService"));
            DeviceManagerProxy device_proxy = new DeviceManagerProxy();
            device_proxy.setEndPoint(new URL("http://localhost:9080/dmserver/services/DeviceManagerService"));

            //**********************************************************************
            //Get all registered notification types
            //**********************************************************************
            String not_types[] = device_proxy.getNotificationTypes();
            System.out.println("\nAll Possible Notification types: ");
            for (int i=0; i<not_types.length; i++) {
               System.out.println("=== " + not_types[i]);
            }

            //**********************************************************************
            //Get all registered notification types for Win32 device class
            //**********************************************************************
            not_types = device_proxy.getDeviceClassNotificationTypes("Win32");
            System.out.println("\n Notification types for Win32 Device Class: ");
            for (int i=0; i<not_types.length; i++) {
               System.out.println("=== " + not_types[i]);
            }

            //**********************************************************************
            //Get notification attribute keys for a valid notification type
            //**********************************************************************
            if (not_types.length >0) {
               String[] not_keys = device_proxy.getNotificationAttributeKeys(not_types[0]);
               System.out.println("\n Notification attribute keys for Notification type - " + not_types[0]);
               for (int i=0; i<not_keys.length; i++) {
                  System.out.println("=== " + not_keys[i]);
               }
            }

            //**********************************************************************
            //Get notification attribute meta-data for a valid notification type.
            //The meta-data describes the attributes.
            //**********************************************************************
            if (not_types.length >0) {
               Element element =  device_proxy.getNotificationAttributeMetaData(not_types[0]);
               System.out.println("\n Notification attribute meta-data for Notification type - " + not_types[0]);
               StringWriter  stringOut = new StringWriter();        //Writer will be a String
               OutputFormat outFormat = new OutputFormat();
               outFormat.setIndenting(true);
               outFormat.setEncoding("UTF-8");
               XMLSerializer serial = new XMLSerializer();
               serial.setOutputCharStream(stringOut);
               serial.setOutputFormat(outFormat);
               serial.asDOMSerializer();                            // As a DOM Serializer
               serial.serialize( element);
               System.out.println(stringOut.toString() ); //Spit out XML as a String
            }

            //**********************************************************************
            //Create a Win32 device, set the Notification type to OSGiHttpPost and
            //set some notification attributes
            //**********************************************************************
            Device device = new Device();
            device.setDeviceClassName("Win32");
            device.setDeviceName("win32b");
            device.setDeviceOwner("TestUser");
            device.setDeviceNotificationType("OSGiHttpPost");

            //Notification attrs
            HashMap map = new HashMap();
            map.put("CLIENT_ADDRESS", "client-host");
            map.put("CLIENT_PORT", "81");
            map.put("CONNECT_TO_DEFAULT_ACCOUNT", "YES");
            device.setDeviceNotificationAttributesMap(map);

            long device_id = device_proxy.createDevice(device);
            System.out.println("Device created with notification attributes, device_id = " + device_id + " Notification attrs: "); 
            System.out.println(map); 

            //**********************************************************************
            //Update device - to remove all notification attributes
            //**********************************************************************
            Device udevice = new Device(); 
            udevice.setDeviceID(device_id);
            udevice.setDeviceNotificationType("");
            device_proxy.updateDevice(udevice);
            System.out.println("Update device to remove all notification attributes"); 

            //**********************************************************************
            //Update device - to reset some notification attributes
            //**********************************************************************
            udevice = new Device(); 
            udevice.setDeviceID(device_id);
            udevice.setDeviceNotificationType("OSGiHttpPost");

            map = new HashMap();
            map.put("CLIENT_ADDRESS", "client-host2");
            map.put("CLIENT_PORT", "81");
            //map.put("CONNECT_TO_DEFAULT_ACCOUNT", "YES");
            udevice.setDeviceNotificationAttributesMap(map);

            device_proxy.updateDevice(udevice);
            System.out.println("Update device to reset notification attributes to:"); 
            System.out.println(map); 

            //Get device to see if only 2 atttributes are set
            udevice = device_proxy.getDevice(device_id); 
            HashMap attrs = udevice.getDeviceNotificationAttributesMap();
            System.out.println("Get device, notification attributes are:"); 
            System.out.println(attrs); 

            return;
        }catch (DeviceManagerException se) {
            System.out.println("API Exception - " + se.getMessage());
        }catch (Exception e) {
            System.out.println("Exception - " + e);
        }
    }
}
